@@ -16,9 +16,9 @@ import logging
 from glob import glob
 from optparse import OptionParser
 from os import remove, mkdir, makedirs
-from os.path import getmtime, join, exists, isdir, relpath, \
+from os.path import join, exists, isdir, relpath, \
                     splitext, normpath, dirname, commonprefix, \
-                    split as path_split
+                    split as path_split, samefile
 from re import split as re_split
 from shutil import copy2, Error as shutil_Error
 import sys
@@ -30,9 +30,19 @@ except ImportError:
     from sha import sha as sha1    # Python 2.4
     from md5 import md5
 
+try:
+    # Python 2.7
+    from collections import OrderedDict  # pylint: disable=E0611
+except ImportError:
+    try:
+        # Python 2.6
+        from odict import odict as OrderedDict
+    except ImportError:
+        pass
+
 logger = logging.getLogger("hashedassets")
 
-__version__ = 0, 2, 1
+__version__ = 0, 2, 2
 
 SERIALIZERS = {}
 
@@ -212,22 +222,26 @@ SERIALIZERS['sed'] = SedSerializer
 class AssetHasher(object):
     def __init__(self, files, output_dir, map_filename, map_name, map_type,
             keep_dirs=False,
+            map_only=False,
+            strip_extensions=False,
             digestlength=9999,  # don't truncate
             hashfun='sha1',
             ):
 
-        self._hash_map = {}  # actually, a map for hashes
+        self.input_dir = dirname(commonprefix(files))
 
-        if not map_type and map_filename:
-            map_type = splitext(map_filename)[1].lstrip(".")
+        self.files = OrderedDict.fromkeys(  # python is lisp!
+                         map(lambda l: relpath(l, self.input_dir),
+                             chain.from_iterable(map(glob, files))))
+
+        logger.debug("Initialized map, is now %s", self.files)
 
         self.output_dir = output_dir
         self.map_filename = map_filename
         self.map_name = map_name
         self.map_type = map_type
-        self.files = chain.from_iterable([glob(path) for path in files])
-        self.input_dir = dirname(commonprefix(files))
-
+        self.map_only = map_only
+        self.strip_extensions = strip_extensions
         self.digestlength = digestlength
         self.keep_dirs = keep_dirs
 
@@ -244,51 +258,64 @@ class AssetHasher(object):
         else:
             fun = {'md5': md5, 'sha1': sha1}[fun]
             hashed_filename = urlsafe_b64encode(
-                    fun(open(filename).read()).digest()).strip("=")\
+                    fun(open(join(self.input_dir, filename)).read()).digest()).strip("=")\
                             [:self.digestlength]
-            if extension:
+            if extension and not self.strip_extensions:
                 hashed_filename = "%s%s" % (hashed_filename, extension)
 
         extra_dirs = ''
 
         if self.keep_dirs:
-            extra_dirs, _ = path_split(relpath(filename, self.input_dir))
+            extra_dirs, _ = path_split(filename)
 
-        return join(self.output_dir, extra_dirs, hashed_filename)
+        return join(extra_dirs, hashed_filename)
 
     def process_file(self, filename):
+        logger.debug("Processing file '%s'", filename)
+
         try:
             hashed_filename = self.digest(self.hashfun, filename)
         except IOError, e:
-            return  # files does not exist, can't be hashed
-
-        mtime = getmtime(filename)
-        map_key = relpath(filename, self.input_dir)
-        map_value = relpath(hashed_filename, self.output_dir)
-
-        if map_key in self._hash_map:
-            old_hashed_filename, _ = self._hash_map[map_key]
-            old_hashed_filename = join(self.output_dir, old_hashed_filename)
-
-            if(hashed_filename == old_hashed_filename):
-                return
-
-            remove(old_hashed_filename)
-            logger.info("rm '%s'", old_hashed_filename)
-            del self._hash_map[map_key]
-
-        # no work to do
-        if map_key in self._hash_map:
+            logger.debug("'%s' does not exist, can't be hashed", filename)
             return
 
+        logger.debug("Determined new hashed filename: '%s'", hashed_filename)
+
+        if self.files[filename]:
+            logger.debug("File has been processed in a previous run (hashed to '%s' then)",
+                    self.files[filename])
+
+            outfile = join(self.output_dir, self.files[filename])
+
+            if exists(outfile):
+                logger.debug("%s still exists", outfile)
+
+                if hashed_filename == self.files[filename]:
+                    # skip file
+                    logger.debug("Skipping file '%s' -> '%s'", filename, self.files[filename])
+                    return
+
+                # remove dangling file
+                if not self.map_only:
+                    remove(outfile)
+                    logger.info("rm '%s'", outfile)
+
+
+        infile = join(self.input_dir, filename)
+        outfile = join(self.output_dir, hashed_filename)
+
         try:
-            copy2(filename, hashed_filename)
-        except shutil_Error, e:
-            if e.args[0].endswith("are the same file"):
-                logger.debug("Won't copy '%s' to itself.", filename)
-                return
-            else:
+            if samefile(infile, outfile):
+               logger.debug("Won't copy '%s' to itself.", filename)
+               return
+        except OSError, e:
+            if not (e.strerror == 'No such file or directory' and e.filename == outfile):
+                assert False,  (dir(e), e.message, e.errno, e.strerror, e.filename)
                 raise
+
+        try:
+            if not self.map_only:
+                copy2(infile, outfile)
         except IOError, e:
             if not e.strerror == 'No such file or directory':
                 raise
@@ -296,15 +323,20 @@ class AssetHasher(object):
             # create parent dirs that are needed for the output file
             logger.debug(hashed_filename)
             create_dir, _ = path_split(hashed_filename)
-            logger.info("mkdir -p %s" % create_dir)
-            makedirs(create_dir)
+            logger.info("mkdir -p %s" % join(self.output_dir, create_dir))
+            makedirs(join(self.output_dir, create_dir))
 
             # try again
-            copy2(filename, hashed_filename)
+            copy2(join(self.input_dir, filename), join(self.output_dir,
+                hashed_filename))
 
-        self._hash_map[map_key] = (map_value, mtime)
+        self.files[filename] = hashed_filename
 
-        logger.info("cp '%s' '%s'", filename, hashed_filename)
+        if not self.map_only:
+            logger.info(
+                "cp '%s' '%s'",
+                join(self.input_dir, filename),
+                join(self.output_dir, hashed_filename))
 
     def process_all_files(self):
         for f in self.files:
@@ -321,26 +353,18 @@ class AssetHasher(object):
 
         deserialized = SERIALIZERS[self.map_type].deserialize(content)
 
-        result = {}
         for filename, hashed_filename in deserialized.iteritems():
-            try:
-                mtime = getmtime(join(self.output_dir, hashed_filename))
-                result[filename] = hashed_filename, mtime
-            except OSError, e:
-                assert 2 == e.errno  # file does not exists, so ignore
+            self.files[filename] = hashed_filename
 
-        self._hash_map = result
+        logger.debug("Read map, is now: %s", self.files)
 
     def write_map(self):
         if not self.map_filename:
             return
 
-        items = dict(
-                  (filename, hashed_filename_mtime[0])
-                  for filename, hashed_filename_mtime
-                  in self._hash_map.iteritems())
-
-        serialized = SERIALIZERS[self.map_type].serialize(items, self.map_name)
+        serialized = SERIALIZERS[self.map_type].serialize(OrderedDict(
+            (k,v) for k, v in self.files.iteritems() if v != None
+            ), self.map_name)
 
         f = open(self.map_filename, "w")
         f.write(serialized)
@@ -351,7 +375,6 @@ class AssetHasher(object):
         self.process_all_files()
         self.write_map()
 
-
 def main(args=None):
     if args == None:
         args = sys.argv[1:]
@@ -443,6 +466,24 @@ def main(args=None):
       help="Don't actually map, keep all file names",
     )
 
+    parser.add_option(
+      "-o",
+      "--map-only",
+      action="store_true",
+      dest="map_only",
+      default=False,
+      help="Don't move files, only generate a map",
+    )
+
+    parser.add_option(
+      "-s",
+      "--strip-extensions",
+      action="store_true",
+      dest="strip_extensions",
+      default=False,
+      help="Strip the file extensions from the hashed files",
+    )
+
     (options, args) = parser.parse_args(args)
 
     if options.identity:
@@ -463,18 +504,32 @@ def main(args=None):
     }.get(options.verbosity, logging.DEBUG)
     logger.setLevel(log_level)
 
-    if len(args) < 3:
-        parser.error("You need to specify at least MAPFILE SOURCE and DEST")
+    if len(args) < 2 and options.map_only:
+        print args
+        parser.error("In --map-only mode, you need to specify at least MAPFILE and SOURCE")
+
+    if len(args) < 3 and not options.map_only:
+        parser.error("You need to specify at least MAPFILE, SOURCE and DEST")
 
     map_filename = args[0]
-    files = args[1:-1]
-    output_dir = normpath(args[-1])
 
-    if not exists(output_dir):
-        mkdir(output_dir)
-        logger.info("mkdir '%s'", output_dir)
-    elif not isdir(output_dir):
-        parser.error("Output dir at '%s' is not a directory" % output_dir)
+    if not options.map_type and map_filename:
+        options.map_type = splitext(map_filename)[1].lstrip(".")
+
+    if not options.map_type in SERIALIZERS.keys():
+        parser.error("Invalid map type: '%s'" % options.map_type)
+
+    if options.map_only:
+        files = args[1:]
+        output_dir = '.'
+    else:
+        files = args[1:-1]
+        output_dir = normpath(args[-1])
+        if not exists(output_dir):
+            mkdir(output_dir)
+            logger.info("mkdir '%s'", output_dir)
+        elif not isdir(output_dir):
+            parser.error("Output dir at '%s' is not a directory" % output_dir)
 
     AssetHasher(
       files=files,
@@ -482,6 +537,8 @@ def main(args=None):
       map_filename=map_filename,
       map_name=options.map_name,
       map_type=options.map_type,
+      map_only=options.map_only,
+      strip_extensions=options.strip_extensions,
       digestlength=options.digestlength,
       hashfun=options.hashfun,
       keep_dirs=options.keep_dirs,
